@@ -15,6 +15,7 @@ static bool eat_file_fd(int, char **, size_t *);
 static bool eat_file_at(int, const char *, char **, size_t *);
 int rematch(const char *, const char *, int);
 static char *rmspace(char *);
+static char *rmspace_len(char *, size_t);
 
 void initialize_portage_env(void);
 void initialize_ebuild_flat(void);
@@ -33,7 +34,7 @@ char pretend = 0;
 char reinitialize = 0;
 char reinitialize_metacache = 0;
 static char *portlogdir;
-static char *portdir;
+static char *main_overlay;
 static char *portarch;
 static char *portvdb;
 static char *portedb;
@@ -72,6 +73,8 @@ void init_coredumps(void)
 /* include common library code */
 #include "libq/libq.c"
 
+static DECLARE_ARRAY(overlays);
+
 _q_static
 void no_colors(void)
 {
@@ -424,19 +427,27 @@ contents_entry *contents_parse_line(char *line)
 /* Handle a single file in the repos.conf format. */
 static void read_one_repos_conf(const char *repos_conf)
 {
+	int nsec;
 	char *conf;
-	const char *repo, *path;
+	const char *main_repo, *repo, *path;
 	dictionary *dict;
 
 	dict = iniparser_load(repos_conf);
 
-	repo = iniparser_getstring(dict, "DEFAULT:main-repo", NULL);
-	if (repo) {
+	main_repo = iniparser_getstring(dict, "DEFAULT:main-repo", NULL);
+
+	nsec = iniparser_getnsec(dict);
+	while (nsec-- > 0) {
+		repo = iniparser_getsecname(dict, nsec);
+		if (!strcmp(repo, "DEFAULT"))
+			continue;
+
 		xasprintf(&conf, "%s:location", repo);
 		path = iniparser_getstring(dict, conf, NULL);
 		if (path) {
-			free(portdir);
-			portdir = xstrdup(path);
+			void *ele = xarraypush_str(overlays, path);
+			if (main_repo && !strcmp(repo, main_repo))
+				main_overlay = ele;
 		}
 		free(conf);
 	}
@@ -460,10 +471,27 @@ static void read_repos_conf(const char *configroot, const char *repos_conf)
 		for (i = 0; i < count; ++i) {
 			const char *name = confs[i]->d_name;
 
-			if (name[0] == '.' || confs[i]->d_type != DT_REG)
+			if (name[0] == '.')
 				continue;
 
+#ifdef DT_UNKNOWN
+			if (confs[i]->d_type != DT_UNKNOWN &&
+			    confs[i]->d_type != DT_REG &&
+			    confs[i]->d_type != DT_LNK)
+				continue;
+#endif
+
 			xasprintf(&sub_conf, "%s/%s", top_conf, name);
+
+#ifdef DT_UNKNOWN
+			if (confs[i]->d_type != DT_REG)
+#endif
+			{
+				struct stat st;
+				if (stat(sub_conf, &st) || S_ISREG(st.st_mode))
+					continue;
+			}
+
 			read_one_repos_conf(sub_conf);
 			free(sub_conf);
 		}
@@ -747,7 +775,7 @@ void initialize_portage_env(void)
 		_Q_EVS(ISTR, FEATURES,            features,            "noman noinfo nodoc")
 		_Q_EVS(STR,  EPREFIX,             eprefix,             CONFIG_EPREFIX)
 		_Q_EVS(STR,  EMERGE_LOG_DIR,      portlogdir,          CONFIG_EPREFIX "var/log")
-		_Q_EVS(STR,  PORTDIR,             portdir,             CONFIG_EPREFIX "usr/portage")
+		_Q_EVS(STR,  PORTDIR,             main_overlay,        CONFIG_EPREFIX "usr/portage")
 		_Q_EVS(STR,  PORTAGE_BINHOST,     binhost,             DEFAULT_PORTAGE_BINHOST)
 		_Q_EVS(STR,  PORTAGE_TMPDIR,      port_tmpdir,         CONFIG_EPREFIX "var/tmp/portage/")
 		_Q_EVS(STR,  PKGDIR,              pkgdir,              CONFIG_EPREFIX "usr/portage/packages/")
@@ -878,6 +906,8 @@ void initialize_portage_env(void)
 	}
 
 	read_repos_conf(configroot, CONFIG_EPREFIX "etc/portage/repos.conf");
+	if (array_cnt(overlays) == 0)
+		xarraypush_str(overlays, main_overlay);
 
 	if (getenv("PORTAGE_QUIET") != NULL)
 		quiet = 1;
@@ -904,7 +934,7 @@ int filter_hidden(const struct dirent *dentry)
 }
 
 static const char *
-initialize_flat(int cache_type, bool force)
+initialize_flat(const char *overlay, int cache_type, bool force)
 {
 	struct dirent **category, **pn, **eb;
 	struct stat st;
@@ -915,7 +945,7 @@ initialize_flat(int cache_type, bool force)
 	int frac, secs, count;
 	FILE *fp;
 
-	xasprintf(&cache_file, "%s/dep/%s/%s", portedb, portdir,
+	xasprintf(&cache_file, "%s/dep/%s/%s", portedb, overlay,
 		(cache_type == CACHE_EBUILD ? ".ebuild.x" : ".metadata.x"));
 
 	/* If we aren't forcing a regen, make sure the file is somewhat sane. */
@@ -930,22 +960,22 @@ initialize_flat(int cache_type, bool force)
 
 	count = frac = secs = 0;
 
-	int portdir_fd, subdir_fd;
-	portdir_fd = open(portdir, O_RDONLY|O_CLOEXEC|O_PATH);
+	int overlay_fd, subdir_fd;
+	overlay_fd = open(overlay, O_RDONLY|O_CLOEXEC|O_PATH);
 
 	if (cache_type == CACHE_METADATA) {
-		subdir_fd = openat(portdir_fd, portcachedir_md5, O_RDONLY|O_CLOEXEC);
+		subdir_fd = openat(overlay_fd, portcachedir_md5, O_RDONLY|O_CLOEXEC);
 		if (subdir_fd == -1) {
-			subdir_fd = openat(portdir_fd, portcachedir_pms, O_RDONLY|O_CLOEXEC);
+			subdir_fd = openat(overlay_fd, portcachedir_pms, O_RDONLY|O_CLOEXEC);
 			if (subdir_fd == -1) {
-				warnp("could not read md5 or pms cache dirs in %s", portdir);
+				warnp("could not read md5 or pms cache dirs in %s", overlay);
 				goto ret;
 			}
 			portcachedir_type = CACHE_METADATA_PMS;
 		} else
 			portcachedir_type = CACHE_METADATA_MD5;
 	} else
-		subdir_fd = portdir_fd;
+		subdir_fd = overlay_fd;
 
 	if ((fp = fopen(cache_file, "we")) == NULL) {
 		warnfp("opening cache failed: %s", cache_file);
@@ -1026,22 +1056,26 @@ initialize_flat(int cache_type, bool force)
 
 	warn("Finished %u entries in %d.%06d seconds", count, secs, frac);
 	if (secs > 120)
-		warn("You should consider using the noatime mount option for PORTDIR='%s' if it's not already enabled", portdir);
+		warn("You should consider using the noatime mount option for '%s' if it's not already enabled", overlay);
 ret:
 	close(subdir_fd);
-	if (subdir_fd != portdir_fd)
-		close(portdir_fd);
+	if (subdir_fd != overlay_fd)
+		close(overlay_fd);
 	return cache_file;
 }
-#define initialize_ebuild_flat() initialize_flat(CACHE_EBUILD, false)
-#define initialize_metadata_flat() initialize_flat(CACHE_METADATA, false)
 
 void reinitialize_as_needed(void)
 {
+	size_t n;
+	const char *overlay;
+
 	if (reinitialize)
-		initialize_flat(CACHE_EBUILD, true);
+		array_for_each(overlays, n, overlay)
+			initialize_flat(overlay, CACHE_EBUILD, true);
+
 	if (reinitialize_metacache)
-		initialize_flat(CACHE_METADATA, true);
+		array_for_each(overlays, n, overlay)
+			initialize_flat(overlay, CACHE_METADATA, true);
 }
 
 typedef struct {
@@ -1359,12 +1393,9 @@ int main(int argc, char **argv)
 	bindtextdomain(argv0, CONFIG_EPREFIX "usr/share/locale");
 	textdomain(argv0);
 
-#if 1
 	if (fstat(fileno(stdout), &st) != -1)
 		if (!isatty(fileno(stdout)))
-			if (S_ISFIFO(st.st_mode) == 0)
-				no_colors();
-#endif
+			no_colors();
 	if ((getenv("TERM") == NULL) || (strcmp(getenv("TERM"), "dumb") == 0))
 		no_colors();
 
