@@ -51,9 +51,10 @@ static const char * const qcache_opts_help[] = {
 /********************************************************************/
 
 typedef struct {
-	char *category;
-	char *package;
-	char *ebuild;
+	const char *overlay;
+	const char *category;
+	const char *package;
+	const char *ebuild;
 	portage_cache *cache_data;
 	unsigned char cur;
 	unsigned char num;
@@ -63,10 +64,11 @@ typedef struct {
 /* Global Variables                                                 */
 /********************************************************************/
 
-static char **archlist; /* Read from PORTDIR/profiles/arch.list in qcache_init() */
+static queue *arches;
 static int archlist_count;
+static size_t arch_longest_len;
 const char status[3] = {'-', '~', '+'};
-int qcache_skip, qcache_test_arch, qcache_last = 0;
+int qcache_skip, qcache_test_arch;
 char *qcache_matchpkg = NULL, *qcache_matchcat = NULL;
 
 /********************************************************************/
@@ -112,6 +114,7 @@ int decode_status(char c)
 _q_static
 int decode_arch(const char *arch)
 {
+	queue *q = arches;
 	int a;
 	const char *p;
 
@@ -119,9 +122,13 @@ int decode_arch(const char *arch)
 	if (*p == '~' || *p == '-')
 		p++;
 
-	for (a = 0; a < archlist_count; ++a)
-		if (strcmp(archlist[a], p) == 0)
+	a = 0;
+	while (q) {
+		if (strcmp(q->name, p) == 0)
 			return a;
+		++a;
+		q = q->next;
+	}
 
 	return -1;
 }
@@ -138,6 +145,7 @@ int decode_arch(const char *arch)
 _q_static
 void print_keywords(const char *category, const char *ebuild, int *keywords)
 {
+	queue *arch = arches;
 	int a;
 	char *package;
 
@@ -148,14 +156,13 @@ void print_keywords(const char *category, const char *ebuild, int *keywords)
 	for (a = 0; a < archlist_count; ++a) {
 		switch (keywords[a]) {
 			case stable:
-				printf("%s%c%s%s ", GREEN, status[keywords[a]], archlist[a], NORM);
+				printf("%s%c%s%s ", GREEN, status[keywords[a]], arch->name, NORM);
 				break;
 			case testing:
-				printf("%s%c%s%s ", YELLOW, status[keywords[a]], archlist[a], NORM);
-				break;
-			default:
+				printf("%s%c%s%s ", YELLOW, status[keywords[a]], arch->name, NORM);
 				break;
 		}
+		arch = arch->next;
 	}
 
 	printf("\n");
@@ -204,105 +211,6 @@ int read_keywords(char *s, int *keywords)
 	return 0;
 }
 
-/********************************************************************/
-/* File reading helper functions                                    */
-/********************************************************************/
-
-/*
- * inline unsigned int qcache_count_lines(char *filename);
- *
- * Count the number of new line characters '\n' in a file.
- *
- * IN:
- *  char *filename - name of the file to read.
- * OUT:
- *  unsigned int count - number of new lines counted.
- * ERR:
- *  -1 is returned if the file cannot be read.
- */
-_q_static
-unsigned int qcache_count_lines(char *filename)
-{
-	int count, fd;
-	char c;
-
-	if ((fd = open(filename, O_RDONLY|O_CLOEXEC)) != -1) {
-		count = 0;
-
-		while (read(fd, &c, 1) == 1)
-			if (c == '\n')
-				count++;
-
-		close(fd);
-		return count;
-	}
-
-	return -1;
-}
-
-/*
- * char **qcache_read_lines(char *filename);
- *
- * Reads in every line contained in a file
- *
- * IN:
- *  char *filename - name of the file to read.
- * OUT:
- *  char **lines - number of new lines counted.
- * ERR:
- *  NULL is returned if an error occurs.
- */
-_q_static
-char **qcache_read_lines(char *filename)
-{
-	int len, fd, count, i, num_lines;
-	char **lines, c;
-
-	if (-1 == (num_lines = qcache_count_lines(filename)))
-		return NULL;
-
-	len   = sizeof(char*) * (num_lines + 1);
-	lines = xzalloc(len);
-
-	if ((fd = open(filename, O_RDONLY|O_CLOEXEC)) != -1) {
-		for (i = 0; i < num_lines; i++) {
-			count = 0;
-
-			/* determine the space needed for storing the line */
-			while (read(fd, &c, 1) == 1 && c != '\n')
-				count++;
-			lseek(fd, (lseek(fd, 0, SEEK_CUR) - count - 1), SEEK_SET);
-
-			lines[i] = xzalloc(sizeof(char) * (count+1));
-
-			/* copy the line into lines[i] */
-			assert(read(fd, lines[i], count) == count);
-			assert(read(fd, &c, 1) == 1);	/* skip '\n' */
-		}
-
-		close(fd);
-		return lines;
-	}
-
-	return NULL;
-}
-
-/*
- * void qcache_free_lines(char **lines);
- *
- * free()'s memory allocated by qcache_read_lines
- */
-_q_static
-void qcache_free_lines(char **lines)
-{
-	int i;
-
-	for (i = 0; lines[i]; i++)
-		free(lines[i]);
-
-	free(lines);
-}
-
 /*
  * portage_cache *qcache_read_cache_file(const char *file);
  *
@@ -319,10 +227,10 @@ _q_static
 portage_cache *qcache_read_cache_file(const char *filename)
 {
 	struct stat s;
-	char *ptr, *buf;
+	char *buf;
 	FILE *f;
 	portage_cache *ret = NULL;
-	size_t len, buflen;
+	size_t len, buflen, linelen;
 
 	if ((f = fopen(filename, "r")) == NULL)
 		goto err;
@@ -336,9 +244,8 @@ portage_cache *qcache_read_cache_file(const char *filename)
 	len = sizeof(*ret) + s.st_size + 1;
 	ret = xzalloc(len);
 
-	while (getline(&buf, &buflen, f) != -1) {
-		if ((ptr = strrchr(buf, '\n')) != NULL)
-			*ptr = 0;
+	while ((linelen = getline(&buf, &buflen, f)) != -1) {
+		rmspace_len(buf, linelen);
 
 		if (strncmp(buf, "DEPEND=", 7) == 0)
 			ret->DEPEND = xstrdup(buf + 7);
@@ -485,6 +392,8 @@ int qcache_ebuild_select(const struct dirent *entry)
 /* Traversal function                                               */
 /********************************************************************/
 
+_q_static void qcache_load_arches(const char *overlay);
+
 /*
  * int qcache_traverse(void (*func)(qcache_data*));
  *
@@ -498,14 +407,16 @@ int qcache_ebuild_select(const struct dirent *entry)
  *  exit or return -1 on failure.
  */
 _q_static
-int qcache_traverse(void (*func)(qcache_data*))
+int qcache_traverse_overlay(void (*func)(qcache_data*), const char *overlay)
 {
-	qcache_data data;
+	qcache_data data = {
+		.overlay = overlay,
+	};
 	char *catpath, *pkgpath, *ebuildpath, *cachepath;
 	int i, j, k, len, num_cat, num_pkg, num_ebuild;
 	struct dirent **categories, **packages, **ebuilds;
 
-	xasprintf(&catpath, "%s/dep/%s", portedb, portdir);
+	xasprintf(&catpath, "%s/dep/%s", portedb, overlay);
 
 	if (-1 == (num_cat = scandir(catpath, &categories, qcache_file_select, alphasort))) {
 		errp("%s", catpath);
@@ -517,7 +428,7 @@ int qcache_traverse(void (*func)(qcache_data*))
 
 	/* traverse categories */
 	for (i = 0; i < num_cat; i++) {
-		xasprintf(&pkgpath, "%s/%s", portdir, categories[i]->d_name);
+		xasprintf(&pkgpath, "%s/%s", overlay, categories[i]->d_name);
 
 		if (-1 == (num_pkg = scandir(pkgpath, &packages, qcache_file_select, alphasort))) {
 			if (errno != ENOENT)
@@ -529,10 +440,8 @@ int qcache_traverse(void (*func)(qcache_data*))
 
 		if (qcache_matchcat) {
 			if (strcmp(categories[i]->d_name, qcache_matchcat) != 0) {
-				for (j = 0; j < num_pkg; j++)
-					free(packages[j]);
+				scandir_free(packages, num_pkg);
 				free(categories[i]);
-				free(packages);
 				free(pkgpath);
 				continue;
 			}
@@ -540,23 +449,21 @@ int qcache_traverse(void (*func)(qcache_data*))
 
 		/* traverse packages */
 		for (j = 0; j < num_pkg; j++) {
-			len = sizeof(char) * (strlen(portdir) + strlen("/") + strlen(categories[i]->d_name) + strlen("/") + strlen(packages[j]->d_name) + 1);
-			ebuildpath = xzalloc(len);
-			snprintf(ebuildpath, len, "%s/%s/%s", portdir, categories[i]->d_name, packages[j]->d_name);
+			xasprintf(&ebuildpath, "%s/%s/%s", overlay, categories[i]->d_name, packages[j]->d_name);
 
 			if (-1 == (num_ebuild = scandir(ebuildpath, &ebuilds, qcache_ebuild_select, qcache_vercmp))) {
-				warnp("%s", ebuildpath);
-				free(packages[i]);
-				free(pkgpath);
+				/* Do not complain about spurious files */
+				if (errno != ENOTDIR)
+					warnp("%s", ebuildpath);
+				free(packages[j]);
+				free(ebuildpath);
 				continue;
 			}
 
 			if (qcache_matchpkg) {
 				if (strcmp(packages[j]->d_name, qcache_matchpkg) != 0) {
-					for (k = 0; k < num_ebuild; k++)
-						free(ebuilds[k]);
+					scandir_free(ebuilds, num_ebuild);
 					free(packages[j]);
-					free(ebuilds);
 					free(ebuildpath);
 					continue;
 				}
@@ -565,31 +472,30 @@ int qcache_traverse(void (*func)(qcache_data*))
 			qcache_skip = 0;
 
 			/* traverse ebuilds */
+			data.num = num_ebuild;
 			for (k = 0; k < num_ebuild; k++) {
-				len = sizeof(char) * (strlen(catpath) + strlen("/") + strlen(categories[i]->d_name) + strlen("/") + strlen(ebuilds[k]->d_name) + 1);
-				cachepath = xzalloc(len);
-				snprintf(cachepath, len, "%s/%s/%s", catpath, categories[i]->d_name, ebuilds[k]->d_name);
-				cachepath[len-8] = '\0'; /* remove ".ebuild" */
+				len = xasprintf(&cachepath, "%s/%s/%s", catpath, categories[i]->d_name, ebuilds[k]->d_name);
+				cachepath[len - 7] = '\0'; /* remove ".ebuild" */
 
 				data.category = categories[i]->d_name;
 				data.package = packages[j]->d_name;
 				data.ebuild = ebuilds[k]->d_name;
 				data.cur = k + 1;
-				data.num = num_ebuild;
 				data.cache_data = qcache_read_cache_file(cachepath);
 
 				if (data.cache_data != NULL) {
-					/* is this the last ebuild? */
-					if (i+1 == num_cat && j+1 == num_pkg && k+1 == num_ebuild)
-						qcache_last = 1;
-
 					if (!qcache_skip)
 						func(&data);
 
 					qcache_free_data(data.cache_data);
-				} else
-					warnp("unable to read cache '%s'\n"
-					      "\tperhaps you need to `egencache -j 4` ?", cachepath);
+				} else {
+					static bool warned = false;
+					if (!warned) {
+						warned = true;
+						warnp("unable to read cache '%s'\n"
+						      "\tperhaps you need to `egencache -j 4` ?", cachepath);
+					}
+				}
 
 				free(ebuilds[k]);
 				free(cachepath);
@@ -611,6 +517,27 @@ int qcache_traverse(void (*func)(qcache_data*))
 	return 0;
 }
 
+_q_static
+int qcache_traverse(void (*func)(qcache_data*))
+{
+	int ret;
+	size_t n;
+	const char *overlay;
+
+	/* Preload all the arches. Not entirely correctly (as arches are bound
+	 * to overlays if set), but oh well. */
+	array_for_each(overlays, n, overlay)
+		qcache_load_arches(overlay);
+
+	ret = 0;
+	array_for_each(overlays, n, overlay)
+		ret |= qcache_traverse_overlay(func, overlay);
+
+	func(NULL);
+
+	return ret;
+}
+
 /********************************************************************/
 /* functors                                                         */
 /********************************************************************/
@@ -621,10 +548,15 @@ void qcache_imlate(qcache_data *data)
 	int *keywords;
 	int a;
 
+	if (!data)
+		return;
+
 	keywords = xmalloc(sizeof(*keywords) * archlist_count);
 
 	if (read_keywords(data->cache_data->KEYWORDS, keywords) < 0) {
-		warn("Failed to read keywords for %s%s/%s%s%s", BOLD, data->category, BLUE, data->ebuild, NORM);
+		if (verbose)
+			warn("Failed to read keywords for %s%s/%s%s%s",
+				BOLD, data->category, BLUE, data->ebuild, NORM);
 		free(keywords);
 		return;
 	}
@@ -652,10 +584,15 @@ void qcache_not(qcache_data *data)
 {
 	int *keywords;
 
+	if (!data)
+		return;
+
 	keywords = xmalloc(sizeof(*keywords) * archlist_count);
 
 	if (read_keywords(data->cache_data->KEYWORDS, keywords) < 0) {
-		warn("Failed to read keywords for %s%s/%s%s%s", BOLD, data->category, BLUE, data->ebuild, NORM);
+		if (verbose)
+			warn("Failed to read keywords for %s%s/%s%s%s",
+				BOLD, data->category, BLUE, data->ebuild, NORM);
 		free(keywords);
 		return;
 	}
@@ -674,10 +611,15 @@ void qcache_all(qcache_data *data)
 {
 	int *keywords;
 
+	if (!data)
+		return;
+
 	keywords = xmalloc(sizeof(*keywords) * archlist_count);
 
 	if (read_keywords(data->cache_data->KEYWORDS, keywords) < 0) {
-		warn("Failed to read keywords for %s%s/%s%s%s", BOLD, data->category, BLUE, data->ebuild, NORM);
+		if (verbose)
+			warn("Failed to read keywords for %s%s/%s%s%s",
+				BOLD, data->category, BLUE, data->ebuild, NORM);
 		free(keywords);
 		return;
 	}
@@ -696,13 +638,18 @@ void qcache_dropped(qcache_data *data)
 	static int possible = 0;
 	int *keywords, i;
 
+	if (!data)
+		return;
+
 	if (data->cur == 1)
 		possible = 0;
 
 	keywords = xmalloc(sizeof(*keywords) * archlist_count);
 
 	if (read_keywords(data->cache_data->KEYWORDS, keywords) < 0) {
-		warn("Failed to read keywords for %s%s/%s%s%s", BOLD, data->category, BLUE, data->ebuild, NORM);
+		if (verbose)
+			warn("Failed to read keywords for %s%s/%s%s%s",
+				BOLD, data->category, BLUE, data->ebuild, NORM);
 		free(keywords);
 		return;
 	}
@@ -735,6 +682,8 @@ _q_static
 void qcache_stats(qcache_data *data)
 {
 	static time_t runtime;
+	static queue *allcats;
+	static const char *last_overlay;
 	static int numpkg  = 0;
 	static int numebld = 0;
 	static int numcat;
@@ -742,25 +691,79 @@ void qcache_stats(qcache_data *data)
 	static int *packages_testing;
 	static int *current_package_keywords;
 	static int *keywords;
-	int a, i;
+	int a;
 
-	if (!numpkg) {
-		struct dirent **categories;
-		char *catpath;
+	/* Is this the last time we'll be called? */
+	if (!data) {
+		const char border[] = "------------------------------------------------------------------";
 
-		xasprintf(&catpath, "%s/dep/%s", portedb, portdir);
+		printf("+%.*s+\n", 25, border);
+		printf("|   general statistics    |\n");
+		printf("+%.*s+\n", 25, border);
+		printf("| %s%13s%s | %s%7d%s |\n", GREEN, "architectures", NORM, BLUE, archlist_count, NORM);
+		printf("| %s%13s%s | %s%7d%s |\n", GREEN, "categories", NORM, BLUE, numcat, NORM);
+		printf("| %s%13s%s | %s%7d%s |\n", GREEN, "packages", NORM, BLUE, numpkg, NORM);
+		printf("| %s%13s%s | %s%7d%s |\n", GREEN, "ebuilds", NORM, BLUE, numebld, NORM);
+		printf("+%.*s+\n\n", 25, border);
 
-		if (-1 == (numcat = scandir(catpath, &categories, qcache_file_select, alphasort))) {
-			errp("%s", catpath);
-			free(catpath);
+		printf("+%.*s+\n", (int)(arch_longest_len + 46), border);
+		printf("|%*skeyword distribution                          |\n",
+			(int)arch_longest_len, "");
+		printf("+%.*s+\n", (int)(arch_longest_len + 46), border);
+		printf("| %s%*s%s |%s%8s%s |%s%8s%s |%s%8s%s | %s%8s%s |\n",
+			RED, (int)arch_longest_len, "architecture", NORM, RED, "stable", NORM,
+			RED, "~arch", NORM, RED, "total", NORM, RED, "total/#pkgs", NORM);
+		printf("| %*s |         |%s%8s%s |         |             |\n",
+			(int)arch_longest_len, "", RED, "only", NORM);
+		printf("+%.*s+\n", (int)(arch_longest_len + 46), border);
+
+		queue *arch = arches;
+		for (a = 0; a < archlist_count; ++a) {
+			printf("| %s%*s%s |", GREEN, (int)arch_longest_len, arch->name, NORM);
+			printf("%s%8d%s |", BLUE, packages_stable[a], NORM);
+			printf("%s%8d%s |", BLUE, packages_testing[a], NORM);
+			printf("%s%8d%s |", BLUE, packages_testing[a]+packages_stable[a], NORM);
+			printf("%s%11.2f%s%% |\n", BLUE, (100.0*(packages_testing[a]+packages_stable[a]))/numpkg, NORM);
+			arch = arch->next;
 		}
 
-		for (i = 0; i < numcat; i++)
-			free(categories[i]);
-		free(categories);
+		printf("+%.*s+\n\n", (int)(arch_longest_len + 46), border);
+
+		printf("Completed in ");
+		print_seconds_for_earthlings(time(NULL) - runtime);
+		printf("\n");
+
+		free(packages_stable);
+		free(packages_testing);
+		free(keywords);
+		free(current_package_keywords);
+		free_sets(allcats);
+		return;
+	}
+
+	if (last_overlay != data->overlay) {
+		DIR *dir;
+		struct dirent *de;
+		char *catpath;
 
 		runtime = time(NULL);
 
+		xasprintf(&catpath, "%s/dep/%s", portedb, data->overlay);
+		dir = opendir(catpath);
+		while ((de = readdir(dir)))
+			if (de->d_type == DT_DIR && de->d_name[0] != '.') {
+				bool ok;
+				allcats = add_set_unique(de->d_name, allcats, &ok);
+				if (ok)
+					++numcat;
+			}
+		closedir(dir);
+		free(catpath);
+
+		last_overlay = data->overlay;
+	}
+
+	if (!numpkg) {
 		packages_stable          = xcalloc(archlist_count, sizeof(*packages_stable));
 		packages_testing         = xcalloc(archlist_count, sizeof(*packages_testing));
 		keywords                 = xcalloc(archlist_count, sizeof(*keywords));
@@ -775,7 +778,9 @@ void qcache_stats(qcache_data *data)
 
 	memset(keywords, 0, archlist_count * sizeof(*keywords));
 	if (read_keywords(data->cache_data->KEYWORDS, keywords) < 0) {
-		warn("Failed to read keywords for %s%s/%s%s%s", BOLD, data->category, BLUE, data->ebuild, NORM);
+		if (verbose)
+			warn("Failed to read keywords for %s%s/%s%s%s",
+				BOLD, data->category, BLUE, data->ebuild, NORM);
 		return;
 	}
 
@@ -805,41 +810,6 @@ void qcache_stats(qcache_data *data)
 			}
 		}
 	}
-
-	if (qcache_last) {
-		printf("+-------------------------+\n");
-		printf("|   general statistics    |\n");
-		printf("+-------------------------+\n");
-		printf("| %s%13s%s | %s%7d%s |\n", GREEN, "architectures", NORM, BLUE, archlist_count, NORM);
-		printf("| %s%13s%s | %s%7d%s |\n", GREEN, "categories", NORM, BLUE, numcat, NORM);
-		printf("| %s%13s%s | %s%7d%s |\n", GREEN, "packages", NORM, BLUE, numpkg, NORM);
-		printf("| %s%13s%s | %s%7d%s |\n", GREEN, "ebuilds", NORM, BLUE, numebld, NORM);
-		printf("+-------------------------+\n\n");
-
-		printf("+----------------------------------------------------------+\n");
-		printf("|                   keyword distribution                   |\n");
-		printf("+----------------------------------------------------------+\n");
-		printf("| %s%12s%s |%s%8s%s |%s%8s%s |%s%8s%s | %s%8s%s |\n", RED, "architecture", NORM, RED, "stable", NORM, RED, "~arch", NORM, RED, "total", NORM, RED, "total/#pkgs", NORM);
-		printf("|              |         |%s%8s%s |         |             |\n", RED, "only", NORM);
-		printf("+----------------------------------------------------------+\n");
-
-		for (a = 0; a < archlist_count; ++a) {
-			printf("| %s%12s%s |", GREEN, archlist[a], NORM);
-			printf("%s%8d%s |", BLUE, packages_stable[a], NORM);
-			printf("%s%8d%s |", BLUE, packages_testing[a], NORM);
-			printf("%s%8d%s |", BLUE, packages_testing[a]+packages_stable[a], NORM);
-			printf("%s%11.2f%s%% |\n", BLUE, (100.0*(packages_testing[a]+packages_stable[a]))/numpkg, NORM);
-		}
-
-		printf("+----------------------------------------------------------+\n\n");
-
-		printf("Completed in %s%d%s seconds.\n", BLUE, (int)(time(NULL)-runtime), NORM);
-
-		free(packages_stable);
-		free(packages_testing);
-		free(keywords);
-		free(current_package_keywords);
-	}
 }
 
 _q_static
@@ -848,13 +818,18 @@ void qcache_testing_only(qcache_data *data)
 	static int possible = 0;
 	int *keywords;
 
+	if (!data)
+		return;
+
 	if (data->cur == 1)
 		possible = 0;
 
 	keywords = xmalloc(sizeof(*keywords) * archlist_count);
 
 	if (read_keywords(data->cache_data->KEYWORDS, keywords) < 0) {
-		warn("Failed to read keywords for %s%s/%s%s%s", BOLD, data->category, BLUE, data->ebuild, NORM);
+		if (verbose)
+			warn("Failed to read keywords for %s%s/%s%s%s",
+				BOLD, data->category, BLUE, data->ebuild, NORM);
 		free(keywords);
 		return;
 	}
@@ -880,39 +855,42 @@ void qcache_testing_only(qcache_data *data)
 /* Misc functions                                                   */
 /********************************************************************/
 
-/*
- * int qcache_init();
- *
- * Initialize variables (archlist, num_arches)
- *
- * OUT:
- *  0 is return on success.
- * ERR:
- *  -1 is returned on error.
- */
 _q_static
-int qcache_init(void)
+void qcache_load_arches(const char *overlay)
 {
-	char *filename;
-	unsigned int len;
+	FILE *fp;
+	char *filename, *s;
+	size_t buflen, linelen;
+	char *buf;
 
-	len      = sizeof(char) * (strlen(portdir) + strlen("/profiles/arch.list") + 1);
-	filename = xzalloc(len);
+	xasprintf(&filename, "%s/profiles/arch.list", overlay);
+	fp = fopen(filename, "re");
+	if (!fp)
+		goto done;
 
-	snprintf(filename, len, "%s/profiles/arch.list", portdir);
+	archlist_count = 0;
+	arch_longest_len = 0;
+	buf = NULL;
+	while ((linelen = getline(&buf, &buflen, fp)) != -1) {
+		rmspace_len(buf, linelen);
 
-	if (NULL == (archlist = qcache_read_lines(filename))) {
-		free(filename);
-		return -1;
+		if ((s = strchr(buf, '#')) != NULL)
+			*s = '\0';
+		if (buf[0] == '\0')
+			continue;
+
+		bool ok;
+		arches = add_set_unique(buf, arches, &ok);
+		if (ok) {
+			++archlist_count;
+			arch_longest_len = MAX(arch_longest_len, strlen(buf));
+		}
 	}
+	free(buf);
 
-	len = 0;
-	while (archlist[len])
-		++len;
-	archlist_count = len;
-
+	fclose(fp);
+ done:
 	free(filename);
-	return 0;
 }
 
 /*
@@ -923,7 +901,7 @@ int qcache_init(void)
 _q_static
 void qcache_free(void)
 {
-	qcache_free_lines(archlist);
+	free_sets(arches);
 }
 
 /********************************************************************/
@@ -956,9 +934,6 @@ int qcache_main(int argc, char **argv)
 		}
 	}
 
-	if (-1 == qcache_init())
-		err("Could not initialize arch list");
-
 	if (optind < argc)
 		qcache_test_arch = decode_arch(argv[optind]);
 
