@@ -78,27 +78,29 @@ print_highlighted_use_flags(char *string, int ind, int argc, char **argv)
 	}
 }
 
-static int
-quse_describe_flag(unsigned int ind, unsigned int argc, char **argv)
+static void
+quse_describe_flag(const char *overlay, unsigned int ind, unsigned int argc, char **argv)
 {
 #define NUM_SEARCH_FILES ARRAY_SIZE(search_files)
-	size_t buflen;
+	size_t buflen, linelen;
 	char *buf, *p;
 	unsigned int i, f;
 	size_t s;
 	const char * const search_files[] = { "use.desc", "use.local.desc", "arch.list", };
 	FILE *fp[NUM_SEARCH_FILES];
+	int dfd, fd;
 	DIR *d;
 	struct dirent *de;
 
 	/* pick 1000 arbitrarily long enough for all files under desc/ */
-	buflen = strlen(portdir) + 1000;
+	buflen = strlen(overlay) + 1000;
 	buf = xmalloc(buflen);
 
 	for (i = 0; i < NUM_SEARCH_FILES; ++i) {
-		snprintf(buf, buflen, "%s/profiles/%s", portdir, search_files[i]);
-		if ((fp[i] = fopen(buf, "r")) == NULL)
-			warnp("skipping %s", search_files[i]);
+		snprintf(buf, buflen, "%s/profiles/%s", overlay, search_files[i]);
+		fp[i] = fopen(buf, "r");
+		if (verbose && fp[i] == NULL)
+			warnp("skipping %s", buf);
 	}
 
 	for (i = ind; i < argc; i++) {
@@ -108,13 +110,11 @@ quse_describe_flag(unsigned int ind, unsigned int argc, char **argv)
 			if (fp[f] == NULL)
 				continue;
 
-			while (getline(&buf, &buflen, fp[f]) != -1) {
-				if (buf[0] == '#' || buf[0] == '\n')
+			while ((linelen = getline(&buf, &buflen, fp[f])) != -1) {
+				rmspace_len(buf, linelen);
+				if (buf[0] == '#' || buf[0] == '\0')
 					continue;
 
-				if ((p = strrchr(buf, '\n')) != NULL)
-					*p = '\0';
-
 				switch (f) {
 					case 0: /* Global use.desc */
 						if (!strncmp(buf, argv[i], s))
@@ -155,9 +155,16 @@ quse_describe_flag(unsigned int ind, unsigned int argc, char **argv)
 			fclose(fp[f]);
 
 	/* now scan the desc dir */
-	snprintf(buf, buflen, "%s/profiles/desc/", portdir);
-	if ((d = opendir(buf)) == NULL) {
-		warnp("skipping profiles/desc/");
+	snprintf(buf, buflen, "%s/profiles/desc/", overlay);
+	dfd = open(buf, O_RDONLY|O_CLOEXEC);
+	if (dfd == -1) {
+		if (verbose)
+			warnp("skipping %s", buf);
+		goto done;
+	}
+	d = fdopendir(dfd);
+	if (!d) {
+		close(dfd);
 		goto done;
 	}
 
@@ -169,22 +176,26 @@ quse_describe_flag(unsigned int ind, unsigned int argc, char **argv)
 		if (strcmp(p, ".desc"))
 			continue;
 
-		snprintf(buf, buflen, "%s/profiles/desc/%s", portdir, de->d_name);
-		if ((fp[0] = fopen(buf, "r")) == NULL) {
-			warn("Could not open '%s' for reading; skipping", de->d_name);
+		fd = openat(dfd, de->d_name, O_RDONLY|O_CLOEXEC);
+		if (fd == -1) {
+			if (verbose)
+				warnp("skipping %s/profiles/desc/%s", overlay, de->d_name);
+			continue;
+		}
+		fp[0] = fdopen(fd, "r");
+		if (!fp[0]) {
+			close(fd);
 			continue;
 		}
 
 		/* Chop the trailing .desc for better display */
 		*p = '\0';
 
-		while (getline(&buf, &buflen, fp[0]) != -1) {
-			if (buf[0] == '#' || buf[0] == '\n')
+		while ((linelen = getline(&buf, &buflen, fp[0])) != -1) {
+			rmspace_len(buf, linelen);
+			if (buf[0] == '#' || buf[0] == '\0')
 				continue;
 
-			if ((p = strrchr(buf, '\n')) != NULL)
-				*p = '\0';
-
 			if ((p = strchr(buf, '-')) == NULL) {
  invalid_line:
 				warn("Invalid line in '%s': %s", de->d_name, buf);
@@ -202,13 +213,12 @@ quse_describe_flag(unsigned int ind, unsigned int argc, char **argv)
 				if (!strcmp(argv[i], buf))
 					printf(" %s%s%s:%s%s%s: %s\n", BOLD, de->d_name, NORM, BLUE, argv[i], NORM, p);
 		}
-		close(f);
+		fclose(fp[0]);
 	}
 	closedir(d);
 
  done:
 	free(buf);
-	return 0;
 }
 
 int quse_main(int argc, char **argv)
@@ -221,7 +231,7 @@ int quse_main(int argc, char **argv)
 	char buf1[_Q_PATH_MAX];
 	char buf2[_Q_PATH_MAX];
 
-	size_t ebuildlen;
+	size_t ebuildlen, linelen;
 	char *ebuild;
 
 	const char *search_var = NULL;
@@ -229,6 +239,8 @@ int quse_main(int argc, char **argv)
 	short quse_all = 0;
 	int regexp_matching = 1, i, idx = 0;
 	size_t search_len;
+	size_t n;
+	const char *overlay;
 
 	DBG("argc=%d argv[0]=%s argv[1]=%s",
 	    argc, argv[0], argc > 1 ? argv[1] : "NULL?");
@@ -248,180 +260,188 @@ int quse_main(int argc, char **argv)
 	if (argc == optind && !quse_all && idx >= 0)
 		quse_usage(EXIT_FAILURE);
 
-	if (idx == -1)
-		return quse_describe_flag(optind, argc, argv);
+	if (idx == -1) {
+		array_for_each(overlays, n, overlay)
+			quse_describe_flag(overlay, optind, argc, argv);
+		return 0;
+	}
 
 	if (quse_all) optind = argc;
-	cache_file = initialize_ebuild_flat();
 
 	search_len = strlen(search_vars[idx]);
 	assert(search_len < sizeof(buf0));
 
-	if ((fp = fopen(cache_file, "r")) == NULL) {
-		warnp("could not read cache: %s", cache_file);
-		return 1;
-	}
+	array_for_each(overlays, n, overlay) {
+		cache_file = initialize_flat(overlay, CACHE_EBUILD, false);
 
-	int portdir_fd = open(portdir, O_RDONLY|O_CLOEXEC|O_PATH);
-
-	ebuild = NULL;
-	while (getline(&ebuild, &ebuildlen, fp) != -1) {
-		FILE *newfp;
-		int fd;
-
-		if ((p = strchr(ebuild, '\n')) != NULL)
-			*p = 0;
-
-		fd = openat(portdir_fd, ebuild, O_RDONLY|O_CLOEXEC);
-		if (fd < 0)
+		if ((fp = fopen(cache_file, "re")) == NULL) {
+			warnp("could not read cache: %s", cache_file);
 			continue;
-		newfp = fdopen(fd, "r");
-		if (newfp != NULL) {
-			unsigned int lineno = 0;
-			char revision[sizeof(buf0)];
-			char date[sizeof(buf0)];
-			char user[sizeof(buf0)];
+		}
 
-			revision[0] = 0;
-			user[0] = 0;
-			date[0] = 0;
-			while (fgets(buf0, sizeof(buf0), newfp) != NULL) {
-				int ok = 0;
-				char warned = 0;
-				lineno++;
+		int overlay_fd = open(overlay, O_RDONLY|O_CLOEXEC|O_PATH);
 
-				if (*buf0 == '#') {
-					if (strncmp(buf0, "# $Header: /", 12) == 0)
-						sscanf(buf0, "%*s %*s %*s %s %s %*s %s %*s %*s", (char *) &revision, (char *) &date, (char *) &user);
-					continue;
-				}
-				if (strncmp(buf0, search_vars[idx], search_len) != 0)
-					continue;
+		ebuild = NULL;
+		while ((linelen = getline(&ebuild, &ebuildlen, fp)) != -1) {
+			FILE *newfp;
+			int fd;
 
-				if ((p = strchr(buf0, '\n')) != NULL)
-					*p = 0;
-				if ((p = strchr(buf0, '#')) != NULL) {
-					if (buf0 != p && p[-1] == ' ')
-						p[-1] = 0;
-					else
-						*p = 0;
-				}
-				if (verbose > 1) {
-					if ((strchr(buf0, '\t') != NULL)
-					    || (strchr(buf0, '$') != NULL)
-					    || (strchr(buf0, '\\') != NULL)
-					    || (strchr(buf0, '\'') != NULL)
-					    || (strstr(buf0, "  ") != NULL)) {
-						warned = 1;
-						warn("# Line %d of %s has an annoying %s", lineno, ebuild, buf0);
-					}
-				}
-#ifdef THIS_SUCKS
-				if ((p = strrchr(&buf0[search_len + 1], '\\')) != NULL) {
+			rmspace_len(ebuild, linelen);
 
-				multiline:
-					*p = ' ';
+			fd = openat(overlay_fd, ebuild, O_RDONLY|O_CLOEXEC);
+			if (fd < 0)
+				continue;
+			newfp = fdopen(fd, "r");
+			if (newfp != NULL) {
+				unsigned int lineno = 0;
+				char revision[sizeof(buf0)];
+				char date[sizeof(buf0)];
+				char user[sizeof(buf0)];
 
-					if (fgets(buf1, sizeof(buf1), newfp) == NULL)
-						continue;
+				revision[0] = 0;
+				user[0] = 0;
+				date[0] = 0;
+				while (fgets(buf0, sizeof(buf0), newfp) != NULL) {
+					int ok = 0;
+					char warned = 0;
 					lineno++;
 
-					if ((p = strchr(buf1, '\n')) != NULL)
-						*p = 0;
-					snprintf(buf2, sizeof(buf2), "%s %s", buf0, buf1);
-					remove_extra_space(buf2);
-					strcpy(buf0, buf2);
-					if ((p = strrchr(buf1, '\\')) != NULL)
-						goto multiline;
-				}
-#else
-				remove_extra_space(buf0);
-#endif
-				while ((p = strrchr(&buf0[search_len + 1], '"')) != NULL)  *p = 0;
-				while ((p = strrchr(&buf0[search_len + 1], '\'')) != NULL) *p = 0;
-				while ((p = strrchr(&buf0[search_len + 1], '\\')) != NULL) *p = ' ';
-
-				if (verbose && warned == 0) {
-					if ((strchr(buf0, '$') != NULL) || (strchr(buf0, '\\') != NULL)) {
-						warned = 1;
-						warn("# Line %d of %s has an annoying %s", lineno, ebuild, buf0);
+					if (*buf0 == '#') {
+						if (strncmp(buf0, "# $Header: /", 12) == 0)
+							sscanf(buf0, "%*s %*s %*s %s %s %*s %s %*s %*s",
+								revision, date, user);
+						continue;
 					}
-				}
+					if (strncmp(buf0, search_vars[idx], search_len) != 0)
+						continue;
 
-				if (strlen(buf0) < search_len + 1) {
-					/* warnf("err '%s'/%zu <= %zu; line %u\n", buf0, strlen(buf0), search_len + 1, lineno); */
-					continue;
-				}
-
-				if ((argc == optind) || (quse_all)) {
-					ok = 1;
-				} else {
-					ok = 0;
-					if (regexp_matching) {
-						for (i = optind; i < argc; ++i) {
-							if (rematch(argv[i], &buf0[search_len + 1], REG_NOSUB) == 0) {
-								ok = 1;
-								break;
-							}
-						}
-					} else {
-						remove_extra_space(buf0);
-						strcpy(buf1, &buf0[search_len + 1]);
-
-						for (i = (size_t) optind; i < argc && argv[i] != NULL; i++) {
-							if (strcmp(buf1, argv[i]) == 0) {
-								ok = 1;
-								break;
-							}
-						}
-						if (ok == 0) while ((p = strchr(buf1, ' ')) != NULL) {
+					if ((p = strchr(buf0, '\n')) != NULL)
+						*p = 0;
+					if ((p = strchr(buf0, '#')) != NULL) {
+						if (buf0 != p && p[-1] == ' ')
+							p[-1] = 0;
+						else
 							*p = 0;
+					}
+					if (verbose > 1) {
+						if ((strchr(buf0, '\t') != NULL)
+						    || (strchr(buf0, '$') != NULL)
+						    || (strchr(buf0, '\\') != NULL)
+						    || (strchr(buf0, '\'') != NULL)
+						    || (strstr(buf0, "  ") != NULL)) {
+							warned = 1;
+							warn("# Line %d of %s has an annoying %s",
+								lineno, ebuild, buf0);
+						}
+					}
+#ifdef THIS_SUCKS
+					if ((p = strrchr(&buf0[search_len + 1], '\\')) != NULL) {
+
+					multiline:
+						*p = ' ';
+
+						if (fgets(buf1, sizeof(buf1), newfp) == NULL)
+							continue;
+						lineno++;
+
+						if ((p = strchr(buf1, '\n')) != NULL)
+							*p = 0;
+						snprintf(buf2, sizeof(buf2), "%s %s", buf0, buf1);
+						remove_extra_space(buf2);
+						strcpy(buf0, buf2);
+						if ((p = strrchr(buf1, '\\')) != NULL)
+							goto multiline;
+					}
+#else
+					remove_extra_space(buf0);
+#endif
+					while ((p = strrchr(&buf0[search_len + 1], '"')) != NULL)  *p = 0;
+					while ((p = strrchr(&buf0[search_len + 1], '\'')) != NULL) *p = 0;
+					while ((p = strrchr(&buf0[search_len + 1], '\\')) != NULL) *p = ' ';
+
+					if (verbose && warned == 0) {
+						if ((strchr(buf0, '$') != NULL) || (strchr(buf0, '\\') != NULL)) {
+							warned = 1;
+							warn("# Line %d of %s has an annoying %s",
+								lineno, ebuild, buf0);
+						}
+					}
+
+					if (strlen(buf0) < search_len + 1) {
+						/* warnf("err '%s'/%zu <= %zu; line %u\n", buf0, strlen(buf0), search_len + 1, lineno); */
+						continue;
+					}
+
+					if ((argc == optind) || (quse_all)) {
+						ok = 1;
+					} else {
+						ok = 0;
+						if (regexp_matching) {
+							for (i = optind; i < argc; ++i) {
+								if (rematch(argv[i], &buf0[search_len + 1], REG_NOSUB) == 0) {
+									ok = 1;
+									break;
+								}
+							}
+						} else {
+							remove_extra_space(buf0);
+							strcpy(buf1, &buf0[search_len + 1]);
+
 							for (i = (size_t) optind; i < argc && argv[i] != NULL; i++) {
 								if (strcmp(buf1, argv[i]) == 0) {
 									ok = 1;
 									break;
 								}
 							}
-							strcpy(buf2, p + 1);
-							strcpy(buf1, buf2);
-							if (strchr(buf1, ' ') == NULL)
+							if (ok == 0) while ((p = strchr(buf1, ' ')) != NULL) {
+								*p = 0;
 								for (i = (size_t) optind; i < argc && argv[i] != NULL; i++) {
-									if (strcmp(buf1, argv[i]) == 0)
+									if (strcmp(buf1, argv[i]) == 0) {
 										ok = 1;
+										break;
+									}
 								}
+								strcpy(buf2, p + 1);
+								strcpy(buf1, buf2);
+								if (strchr(buf1, ' ') == NULL)
+									for (i = (size_t) optind; i < argc && argv[i] != NULL; i++) {
+										if (strcmp(buf1, argv[i]) == 0)
+											ok = 1;
+									}
+							}
 						}
 					}
-				}
-				if (ok) {
-					if (verbose > 3)
-						printf("%s %s %s ", *user ? user : "MISSING", *revision ? revision : "MISSING", *date ? date : "MISSING");
+					if (ok) {
+						if (verbose > 3)
+							printf("%s %s %s ",
+								*user ? user : "MISSING",
+								*revision ? revision : "MISSING",
+								*date ? date : "MISSING");
 
-					printf("%s%s%s ", CYAN, ebuild, NORM);
-					print_highlighted_use_flags(&buf0[search_len + 1], optind, argc, argv);
-					puts(NORM);
-					if (verbose > 1) {
-						char **ARGV = NULL;
-						int ARGC = 0;
-						makeargv(&buf0[search_len + 1], &ARGC, &ARGV);
-						if (ARGC > 0) {
-							quse_describe_flag(1, ARGC, ARGV);
-							for (i = 0; i < ARGC; i++)
-								free(ARGV[i]);
-							free(ARGV);
+						printf("%s%s%s ", CYAN, ebuild, NORM);
+						print_highlighted_use_flags(&buf0[search_len + 1], optind, argc, argv);
+						puts(NORM);
+						if (verbose > 1) {
+							char **ARGV;
+							int ARGC;
+							makeargv(&buf0[search_len + 1], &ARGC, &ARGV);
+							quse_describe_flag(overlay, 1, ARGC, ARGV);
+							freeargv(ARGC, ARGV);
 						}
 					}
+					break;
 				}
-				break;
+				fclose(newfp);
+			} else {
+				if (!reinitialize)
+					warnfp("(cache update pending) %s", ebuild);
+				reinitialize = 1;
 			}
-			fclose(newfp);
-		} else {
-			if (!reinitialize)
-				warnfp("(cache update pending) %s", ebuild);
-			reinitialize = 1;
 		}
+		fclose(fp);
+		close(overlay_fd);
 	}
-	fclose(fp);
-	close(portdir_fd);
+
 	return EXIT_SUCCESS;
 }
 
