@@ -33,6 +33,8 @@ setup_env
 unset ROOT PORTAGE_CONFIGROOT PORTAGE_QUIET
 # but make sure we don't implicitly rely on user's setup
 export PORTAGE_CONFIGROOT="${ab}/not/a/real/path"
+# Always use UTC for timestamps to keep tests stable. #551806
+export TZ='UTC 0'
 
 q -i -q
 
@@ -94,3 +96,16 @@ treedir() {
 		ls -R "${d}"
 	fi
 }
+
+mkportdir() {
+	export PORTDIR=$1
+
+	export PORTAGE_CONFIGROOT=${PWD}
+	mkdir -p etc/portage/
+	cat <<-EOF >etc/portage/repos.conf
+	[DEFAULT]
+	main-repo = utils-test
+	[utils-test]
+	location = ${PORTDIR}
+	EOF
+}
